# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/servergroupservice/server_group_service__client.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from tecton_proto.auth import service__client_pb2 as tecton__proto_dot_auth_dot_service____client__pb2
from tecton_proto.common import server_group_status__client_pb2 as tecton__proto_dot_common_dot_server__group__status____client__pb2
from tecton_proto.common import server_group_type__client_pb2 as tecton__proto_dot_common_dot_server__group__type____client__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nBtecton_proto/servergroupservice/server_group_service__client.proto\x12\x1ftecton_proto.servergroupservice\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\'tecton_proto/auth/service__client.proto\x1a\x35tecton_proto/common/server_group_status__client.proto\x1a\x33tecton_proto/common/server_group_type__client.proto\"q\n\x17ListServerGroupsRequest\x12\x1c\n\tworkspace\x18\x01 \x01(\tR\tworkspace\x12\x38\n\x04type\x18\x02 \x01(\x0e\x32$.tecton_proto.common.ServerGroupTypeR\x04type\"q\n\x18ListServerGroupsResponse\x12U\n\rserver_groups\x18\x01 \x03(\x0b\x32\x30.tecton_proto.servergroupservice.ServerGroupInfoR\x0cserverGroups\"a\n\x15GetServerGroupRequest\x12\x1c\n\tworkspace\x18\x01 \x01(\tR\tworkspace\x12*\n\x11server_group_name\x18\x02 \x01(\tR\x0fserverGroupName\"m\n\x16GetServerGroupResponse\x12S\n\x0cserver_group\x18\x01 \x01(\x0b\x32\x30.tecton_proto.servergroupservice.ServerGroupInfoR\x0bserverGroup\"\xfa\x05\n\x0fServerGroupInfo\x12&\n\x0fserver_group_id\x18\x01 \x01(\tR\rserverGroupId\x12\x12\n\x04name\x18\x02 \x01(\tR\x04name\x12\x38\n\x04type\x18\x03 \x01(\x0e\x32$.tecton_proto.common.ServerGroupTypeR\x04type\x12 \n\x0b\x64\x65scription\x18\x04 \x01(\tR\x0b\x64\x65scription\x12\x39\n\ncreated_at\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tcreatedAt\x12\x14\n\x05owner\x18\x06 \x01(\tR\x05owner\x12(\n\x10last_modified_by\x18\x07 \x01(\tR\x0elastModifiedBy\x12N\n\x04tags\x18\x08 \x03(\x0b\x32:.tecton_proto.servergroupservice.ServerGroupInfo.TagsEntryR\x04tags\x12`\n\x0e\x64\x65sired_config\x18\n \x01(\x0b\x32\x39.tecton_proto.servergroupservice.ServerGroupScalingConfigR\rdesiredConfig\x12`\n\x0e\x63urrent_config\x18\x0b \x01(\x0b\x32\x39.tecton_proto.servergroupservice.ServerGroupScalingConfigR\rcurrentConfig\x12>\n\x06status\x18\x0c \x01(\x0e\x32&.tecton_proto.common.ServerGroupStatusR\x06status\x12%\n\x0estatus_details\x18\r \x01(\tR\rstatusDetails\x12 \n\x0b\x65nvironment\x18\x0e \x01(\tR\x0b\x65nvironment\x1a\x37\n\tTagsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\"\x9c\x02\n\x18ServerGroupScalingConfig\x12\x1b\n\tmin_nodes\x18\x01 \x01(\x05R\x08minNodes\x12\x1b\n\tmax_nodes\x18\x02 \x01(\x05R\x08maxNodes\x12#\n\rdesired_nodes\x18\x03 \x01(\x05R\x0c\x64\x65siredNodes\x12/\n\x13\x61utoscaling_enabled\x18\x04 \x01(\x08R\x12\x61utoscalingEnabled\x12\x42\n\x0flast_updated_at\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\rlastUpdatedAt\x12,\n\x12workspace_state_id\x18\x06 \x01(\tR\x10workspaceStateId2\x80\x03\n\x12ServerGroupService\x12\xb6\x01\n\x10ListServerGroups\x12\x38.tecton_proto.servergroupservice.ListServerGroupsRequest\x1a\x39.tecton_proto.servergroupservice.ListServerGroupsResponse\"-\xa2\xbc\xe6\xc0\x05\'*%tecton/organization#read_organization\x12\xb0\x01\n\x0eGetServerGroup\x12\x36.tecton_proto.servergroupservice.GetServerGroupRequest\x1a\x37.tecton_proto.servergroupservice.GetServerGroupResponse\"-\xa2\xbc\xe6\xc0\x05\'*%tecton/organization#read_organizationBp\n\x1d\x63om.tecton.servergroupserviceB\x17ServerGroupServiceProtoP\x01Z4github.com/tecton-ai/tecton_proto/servergroupservice')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.servergroupservice.server_group_service__client_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\035com.tecton.servergroupserviceB\027ServerGroupServiceProtoP\001Z4github.com/tecton-ai/tecton_proto/servergroupservice'
  _SERVERGROUPINFO_TAGSENTRY._options = None
  _SERVERGROUPINFO_TAGSENTRY._serialized_options = b'8\001'
  _SERVERGROUPSERVICE.methods_by_name['ListServerGroups']._options = None
  _SERVERGROUPSERVICE.methods_by_name['ListServerGroups']._serialized_options = b'\242\274\346\300\005\'*%tecton/organization#read_organization'
  _SERVERGROUPSERVICE.methods_by_name['GetServerGroup']._options = None
  _SERVERGROUPSERVICE.methods_by_name['GetServerGroup']._serialized_options = b'\242\274\346\300\005\'*%tecton/organization#read_organization'
  _LISTSERVERGROUPSREQUEST._serialized_start=285
  _LISTSERVERGROUPSREQUEST._serialized_end=398
  _LISTSERVERGROUPSRESPONSE._serialized_start=400
  _LISTSERVERGROUPSRESPONSE._serialized_end=513
  _GETSERVERGROUPREQUEST._serialized_start=515
  _GETSERVERGROUPREQUEST._serialized_end=612
  _GETSERVERGROUPRESPONSE._serialized_start=614
  _GETSERVERGROUPRESPONSE._serialized_end=723
  _SERVERGROUPINFO._serialized_start=726
  _SERVERGROUPINFO._serialized_end=1488
  _SERVERGROUPINFO_TAGSENTRY._serialized_start=1433
  _SERVERGROUPINFO_TAGSENTRY._serialized_end=1488
  _SERVERGROUPSCALINGCONFIG._serialized_start=1491
  _SERVERGROUPSCALINGCONFIG._serialized_end=1775
  _SERVERGROUPSERVICE._serialized_start=1778
  _SERVERGROUPSERVICE._serialized_end=2162
# @@protoc_insertion_point(module_scope)
