from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from typing import ClassVar

DESCRIPTOR: _descriptor.FileDescriptor
SERVER_GROUP_TYPE_FEATURE_SERVER_GROUP: ServerGroupType
SERVER_GROUP_TYPE_TRANSFORM_SERVER_GROUP: ServerGroupType
SERVER_GROUP_TYPE_UNSPECIFIED: ServerGroupType

class ServerGroupType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []
