# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/realtime/instance_group__client.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tecton_proto.common import container_image__client_pb2 as tecton__proto_dot_common_dot_container__image____client__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n2tecton_proto/realtime/instance_group__client.proto\x12\x15tecton_proto.realtime\x1a\x31tecton_proto/common/container_image__client.proto\"\xb0\x08\n\rInstanceGroup\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\x12L\n\x0f\x63ontainer_image\x18\x02 \x01(\x0b\x32#.tecton_proto.common.ContainerImageR\x0e\x63ontainerImage\x12\x41\n\x08\x63\x61pacity\x18\x03 \x01(\x0b\x32%.tecton_proto.realtime.CapacityConfigR\x08\x63\x61pacity\x12Z\n\x13health_check_config\x18\x04 \x01(\x0b\x32(.tecton_proto.realtime.HealthCheckConfigH\x00R\x11healthCheckConfig\x12,\n\x11health_check_name\x18\x0c \x01(\tH\x00R\x0fhealthCheckName\x12I\n\x0fprometheus_port\x18\x05 \x01(\x0b\x32 .tecton_proto.realtime.NamedPortR\x0eprometheusPort\x12=\n\tgrpc_port\x18\x06 \x01(\x0b\x32 .tecton_proto.realtime.NamedPortR\x08grpcPort\x12=\n\thttp_port\x18\x0b \x01(\x0b\x32 .tecton_proto.realtime.NamedPortR\x08httpPort\x12W\n\x12\x61ws_instance_group\x18\x07 \x01(\x0b\x32\'.tecton_proto.realtime.AWSInstanceGroupH\x01R\x10\x61wsInstanceGroup\x12p\n\x1bgoogle_cloud_instance_group\x18\x08 \x01(\x0b\x32/.tecton_proto.realtime.GoogleCloudInstanceGroupH\x01R\x18googleCloudInstanceGroup\x12\x42\n\x04tags\x18\t \x03(\x0b\x32..tecton_proto.realtime.InstanceGroup.TagsEntryR\x04tags\x12s\n\x15\x65nvironment_variables\x18\n \x03(\x0b\x32>.tecton_proto.realtime.InstanceGroup.EnvironmentVariablesEntryR\x14\x65nvironmentVariables\x1a\x37\n\tTagsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\x1aG\n\x19\x45nvironmentVariablesEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\x42\x13\n\x11health_check_typeB\x0c\n\ncloud_type\"\xae\x01\n\x13InstanceGroupHandle\x12*\n\x11instance_group_id\x18\x01 \x01(\tR\x0finstanceGroupId\x12.\n\x13instance_group_name\x18\x02 \x01(\tR\x11instanceGroupName\x12;\n\x1ainstance_group_template_id\x18\x03 \x01(\tR\x17instanceGroupTemplateId\"\xa0\x01\n\x0e\x43\x61pacityConfig\x12/\n\x13\x61utoscaling_enabled\x18\x04 \x01(\x08R\x12\x61utoscalingEnabled\x12\x1b\n\tmin_nodes\x18\x01 \x01(\x05R\x08minNodes\x12\x1b\n\tmax_nodes\x18\x02 \x01(\x05R\x08maxNodes\x12#\n\rdesired_nodes\x18\x03 \x01(\x05R\x0c\x64\x65siredNodes\"B\n\x13InstanceGroupStatus\x12+\n\x11healthy_instances\x18\x01 \x01(\x05R\x10healthyInstances\"\x82\x01\n\x11HealthCheckConfig\x12\x34\n\x04port\x18\x01 \x01(\x0b\x32 .tecton_proto.realtime.NamedPortR\x04port\x12\x12\n\x04path\x18\x02 \x01(\tR\x04path\x12#\n\rprotocol_type\x18\x03 \x01(\tR\x0cprotocolType\"I\n\tNamedPort\x12\x1f\n\x0bport_number\x18\x01 \x01(\x05R\nportNumber\x12\x1b\n\tport_name\x18\x02 \x01(\tR\x08portName\"\xcf\x03\n\x10\x41WSInstanceGroup\x12\x32\n\x15\x61utoscaling_group_arn\x18\x01 \x01(\tR\x13\x61utoscalingGroupArn\x12\x34\n\x16\x61utoscaling_group_name\x18\x02 \x01(\tR\x14\x61utoscalingGroupName\x12\x16\n\x06region\x18\x03 \x01(\tR\x06region\x12\x12\n\x04port\x18\x04 \x01(\x05R\x04port\x12*\n\x11health_check_path\x18\x05 \x01(\tR\x0fhealthCheckPath\x12#\n\rinstance_type\x18\x06 \x01(\tR\x0cinstanceType\x12 \n\x0c\x61mi_image_id\x18\x07 \x01(\tR\namiImageId\x12\x37\n\x18iam_instance_profile_arn\x18\x08 \x01(\tR\x15iamInstanceProfileArn\x12,\n\x12security_group_ids\x18\t \x03(\tR\x10securityGroupIds\x12\x1d\n\nsubnet_ids\x18\n \x03(\tR\tsubnetIds\x12,\n\x12launch_template_id\x18\x0b \x01(\tR\x10launchTemplateId\"e\n\x1c\x41WSInstanceGroupUpdateConfig\x12#\n\rinstance_type\x18\x01 \x01(\tR\x0cinstanceType\x12 \n\x0c\x61mi_image_id\x18\x02 \x01(\tR\namiImageId\"\x86\x01\n\x0e\x41WSTargetGroup\x12\x10\n\x03\x61rn\x18\x01 \x01(\tR\x03\x61rn\x12\x12\n\x04name\x18\x02 \x01(\tR\x04name\x12N\n\x0einstance_group\x18\x03 \x01(\x0b\x32\'.tecton_proto.realtime.AWSInstanceGroupR\rinstanceGroup\"\xda\x01\n\x18GoogleCloudInstanceGroup\x12\x18\n\x07project\x18\x01 \x01(\tR\x07project\x12\x16\n\x06region\x18\x02 \x01(\tR\x06region\x12\x1b\n\ttarget_id\x18\x03 \x01(\tR\x08targetId\x12!\n\x0cmachine_type\x18\x04 \x01(\tR\x0bmachineType\x12 \n\x0bsubnetworks\x18\x05 \x03(\tR\x0bsubnetworks\x12*\n\x11health_check_name\x18\x06 \x01(\tR\x0fhealthCheckName\"\xc2\x01\n\x19GoogleCloudBackendService\x12\x1b\n\ttarget_id\x18\x01 \x01(\tR\x08targetId\x12\x18\n\x07project\x18\x02 \x01(\tR\x07project\x12\x16\n\x06region\x18\x03 \x01(\tR\x06region\x12V\n\x0einstance_group\x18\x04 \x01(\x0b\x32/.tecton_proto.realtime.GoogleCloudInstanceGroupR\rinstanceGroup\"\xdb\x01\n\x12LoadBalancerTarget\x12Q\n\x10\x61ws_target_group\x18\x01 \x01(\x0b\x32%.tecton_proto.realtime.AWSTargetGroupH\x00R\x0e\x61wsTargetGroup\x12h\n\x16google_backend_service\x18\x02 \x01(\x0b\x32\x30.tecton_proto.realtime.GoogleCloudBackendServiceH\x00R\x14googleBackendServiceB\x08\n\x06targetB\x1c\n\x18\x63om.tecton.realtimeprotoP\x01')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.realtime.instance_group__client_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\030com.tecton.realtimeprotoP\001'
  _INSTANCEGROUP_TAGSENTRY._options = None
  _INSTANCEGROUP_TAGSENTRY._serialized_options = b'8\001'
  _INSTANCEGROUP_ENVIRONMENTVARIABLESENTRY._options = None
  _INSTANCEGROUP_ENVIRONMENTVARIABLESENTRY._serialized_options = b'8\001'
  _INSTANCEGROUP._serialized_start=129
  _INSTANCEGROUP._serialized_end=1201
  _INSTANCEGROUP_TAGSENTRY._serialized_start=1038
  _INSTANCEGROUP_TAGSENTRY._serialized_end=1093
  _INSTANCEGROUP_ENVIRONMENTVARIABLESENTRY._serialized_start=1095
  _INSTANCEGROUP_ENVIRONMENTVARIABLESENTRY._serialized_end=1166
  _INSTANCEGROUPHANDLE._serialized_start=1204
  _INSTANCEGROUPHANDLE._serialized_end=1378
  _CAPACITYCONFIG._serialized_start=1381
  _CAPACITYCONFIG._serialized_end=1541
  _INSTANCEGROUPSTATUS._serialized_start=1543
  _INSTANCEGROUPSTATUS._serialized_end=1609
  _HEALTHCHECKCONFIG._serialized_start=1612
  _HEALTHCHECKCONFIG._serialized_end=1742
  _NAMEDPORT._serialized_start=1744
  _NAMEDPORT._serialized_end=1817
  _AWSINSTANCEGROUP._serialized_start=1820
  _AWSINSTANCEGROUP._serialized_end=2283
  _AWSINSTANCEGROUPUPDATECONFIG._serialized_start=2285
  _AWSINSTANCEGROUPUPDATECONFIG._serialized_end=2386
  _AWSTARGETGROUP._serialized_start=2389
  _AWSTARGETGROUP._serialized_end=2523
  _GOOGLECLOUDINSTANCEGROUP._serialized_start=2526
  _GOOGLECLOUDINSTANCEGROUP._serialized_end=2744
  _GOOGLECLOUDBACKENDSERVICE._serialized_start=2747
  _GOOGLECLOUDBACKENDSERVICE._serialized_end=2941
  _LOADBALANCERTARGET._serialized_start=2944
  _LOADBALANCERTARGET._serialized_end=3163
# @@protoc_insertion_point(module_scope)
