# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/data/realtime_compute__client.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tecton_proto.realtime import instance_group__client_pb2 as tecton__proto_dot_realtime_dot_instance__group____client__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n0tecton_proto/data/realtime_compute__client.proto\x12\x11tecton_proto.data\x1a\x32tecton_proto/realtime/instance_group__client.proto\"\x18\n\x16\x43olocatedComputeConfig\"d\n\x1bRemoteFunctionComputeConfig\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\x12\x12\n\x04name\x18\x02 \x01(\tR\x04name\x12!\n\x0c\x66unction_uri\x18\x03 \x01(\tR\x0b\x66unctionUri\"\x8e\x01\n\x1aInstanceGroupComputeConfig\x12\x1d\n\ngroup_name\x18\x01 \x01(\tR\tgroupName\x12Q\n\x0einstance_group\x18\x02 \x01(\x0b\x32*.tecton_proto.realtime.InstanceGroupHandleR\rinstanceGroup\"\xb8\x02\n\x13OnlineComputeConfig\x12X\n\x11\x63olocated_compute\x18\x01 \x01(\x0b\x32).tecton_proto.data.ColocatedComputeConfigH\x00R\x10\x63olocatedCompute\x12W\n\x0eremote_compute\x18\x02 \x01(\x0b\x32..tecton_proto.data.RemoteFunctionComputeConfigH\x00R\rremoteCompute\x12\x63\n\x15instance_group_config\x18\x03 \x01(\x0b\x32-.tecton_proto.data.InstanceGroupComputeConfigH\x00R\x13instanceGroupConfigB\t\n\x07\x63omputeB\x13\n\x0f\x63om.tecton.dataP\x01')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.data.realtime_compute__client_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\017com.tecton.dataP\001'
  _COLOCATEDCOMPUTECONFIG._serialized_start=123
  _COLOCATEDCOMPUTECONFIG._serialized_end=147
  _REMOTEFUNCTIONCOMPUTECONFIG._serialized_start=149
  _REMOTEFUNCTIONCOMPUTECONFIG._serialized_end=249
  _INSTANCEGROUPCOMPUTECONFIG._serialized_start=252
  _INSTANCEGROUPCOMPUTECONFIG._serialized_end=394
  _ONLINECOMPUTECONFIG._serialized_start=397
  _ONLINECOMPUTECONFIG._serialized_end=709
# @@protoc_insertion_point(module_scope)
