# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/common/server_group_status__client.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n5tecton_proto/common/server_group_status__client.proto\x12\x13tecton_proto.common*\xfd\x01\n\x11ServerGroupStatus\x12#\n\x1fSERVER_GROUP_STATUS_UNSPECIFIED\x10\x00\x12 \n\x1cSERVER_GROUP_STATUS_CREATING\x10\x01\x12\x1f\n\x1bSERVER_GROUP_STATUS_PENDING\x10\x06\x12\x1d\n\x19SERVER_GROUP_STATUS_READY\x10\x02\x12 \n\x1cSERVER_GROUP_STATUS_UPDATING\x10\x03\x12\x1d\n\x19SERVER_GROUP_STATUS_ERROR\x10\x04\x12 \n\x1cSERVER_GROUP_STATUS_DELETING\x10\x05\x42\x15\n\x11\x63om.tecton.commonP\x01')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.common.server_group_status__client_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\021com.tecton.commonP\001'
  _SERVERGROUPSTATUS._serialized_start=79
  _SERVERGROUPSTATUS._serialized_end=332
# @@protoc_insertion_point(module_scope)
