BUILD_STATUS = {'BUILD_ID': 'uncommitted-buildkite-agent-1730316131', 'BUILD_TIMESTAMP': '1730316131', 'FORMATTED_DATE': '2024 Oct 30 19 22 11 Wed', 'GIT_COMMIT': '0289549850ef55f040ea02da5144e54687aeef58', 'BUILD_EMBED_LABEL': '', 'BUILD_HOST': 'buildkite-agent-medium-6cf65dcd96-66qd6', 'BUILD_USER': 'buildkite-agent'}
