# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/data/server_group__client.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tecton_proto.common import container_image__client_pb2 as tecton__proto_dot_common_dot_container__image____client__pb2
from tecton_proto.common import id__client_pb2 as tecton__proto_dot_common_dot_id____client__pb2
from tecton_proto.common import scaling_config__client_pb2 as tecton__proto_dot_common_dot_scaling__config____client__pb2
from tecton_proto.common import server_group_type__client_pb2 as tecton__proto_dot_common_dot_server__group__type____client__pb2
from tecton_proto.data import fco_metadata__client_pb2 as tecton__proto_dot_data_dot_fco__metadata____client__pb2
from tecton_proto.validation import validator__client_pb2 as tecton__proto_dot_validation_dot_validator____client__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n,tecton_proto/data/server_group__client.proto\x12\x11tecton_proto.data\x1a\x31tecton_proto/common/container_image__client.proto\x1a$tecton_proto/common/id__client.proto\x1a\x30tecton_proto/common/scaling_config__client.proto\x1a\x33tecton_proto/common/server_group_type__client.proto\x1a,tecton_proto/data/fco_metadata__client.proto\x1a/tecton_proto/validation/validator__client.proto\"\xee\x06\n\x0bServerGroup\x12?\n\x0fserver_group_id\x18\x01 \x01(\x0b\x32\x17.tecton_proto.common.IdR\rserverGroupId\x12\x41\n\x0c\x66\x63o_metadata\x18\x02 \x01(\x0b\x32\x1e.tecton_proto.data.FcoMetadataR\x0b\x66\x63oMetadata\x12\x38\n\x04type\x18\x03 \x01(\x0e\x32$.tecton_proto.common.ServerGroupTypeR\x04type\x12_\n\x16transform_server_group\x18\x08 \x01(\x0b\x32\'.tecton_proto.data.TransformServerGroupH\x00R\x14transformServerGroup\x12Y\n\x14\x66\x65\x61ture_server_group\x18\n \x01(\x0b\x32%.tecton_proto.data.FeatureServerGroupH\x00R\x12\x66\x65\x61tureServerGroup\x12\x45\n\x07options\x18\t \x03(\x0b\x32+.tecton_proto.data.ServerGroup.OptionsEntryR\x07options\x12[\n\x0fvalidation_args\x18\x0b \x01(\x0b\x32\x32.tecton_proto.validation.ServerGroupValidationArgsR\x0evalidationArgs\x12W\n\x12\x61utoscaling_config\x18\x0c \x01(\x0b\x32&.tecton_proto.common.AutoscalingConfigH\x01R\x11\x61utoscalingConfig\x12m\n\x1aprovisioned_scaling_config\x18\r \x01(\x0b\x32-.tecton_proto.common.ProvisionedScalingConfigH\x01R\x18provisionedScalingConfig\x1a:\n\x0cOptionsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\x42\x13\n\x11server_group_typeB\x10\n\x0escaling_configJ\x04\x08\x04\x10\x05J\x04\x08\x05\x10\x06J\x04\x08\x06\x10\x07J\x04\x08\x07\x10\x08\"\xac\x01\n\x14TransformServerGroup\x12%\n\x0e\x65nvironment_id\x18\x01 \x01(\tR\renvironmentId\x12)\n\x10\x65nvironment_name\x18\x02 \x01(\tR\x0f\x65nvironmentName\x12\x42\n\nimage_info\x18\x03 \x01(\x0b\x32#.tecton_proto.common.ContainerImageR\timageInfo\"\x14\n\x12\x46\x65\x61tureServerGroupB\x13\n\x0f\x63om.tecton.dataP\x01')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.data.server_group__client_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\017com.tecton.dataP\001'
  _SERVERGROUP_OPTIONSENTRY._options = None
  _SERVERGROUP_OPTIONSENTRY._serialized_options = b'8\001'
  _SERVERGROUP._serialized_start=355
  _SERVERGROUP._serialized_end=1233
  _SERVERGROUP_OPTIONSENTRY._serialized_start=1112
  _SERVERGROUP_OPTIONSENTRY._serialized_end=1170
  _TRANSFORMSERVERGROUP._serialized_start=1236
  _TRANSFORMSERVERGROUP._serialized_end=1408
  _FEATURESERVERGROUP._serialized_start=1410
  _FEATURESERVERGROUP._serialized_end=1430
# @@protoc_insertion_point(module_scope)
