from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from typing import ClassVar

COPIER_STATUS_ERROR: OnlineStoreCopierTaskState
COPIER_STATUS_PENDING: OnlineStoreCopierTaskState
COPIER_STATUS_RUNNING: OnlineStoreCopierTaskState
COPIER_STATUS_SUCCESS: OnlineStoreCopierTaskState
COPIER_STATUS_UNSPECIFIED: OnlineStoreCopierTaskState
DESCRIPTOR: _descriptor.FileDescriptor
MATERIALIZATION_ATTEMPT_STATUS_CANCELLED: MaterializationTaskAttemptState
MATERIALIZATION_ATTEMPT_STATUS_ERROR: MaterializationTaskAttemptState
MATERIALIZATION_ATTEMPT_STATUS_PENDING: MaterializationTaskAttemptState
MATERIALIZATION_ATTEMPT_STATUS_RUNNING: MaterializationTaskAttemptState
MATERIALIZATION_ATTEMPT_STATUS_SUCCESS: MaterializationTaskAttemptState
MATERIALIZATION_ATTEMPT_STATUS_UNSPECIFIED: MaterializationTaskAttemptState
MATERIALIZATION_TASK_STATUS_DRAINED: MaterializationTaskState
MATERIALIZATION_TASK_STATUS_DRAINING: MaterializationTaskState
MATERIALIZATION_TASK_STATUS_MANUALLY_CANCELLED: MaterializationTaskState
MATERIALIZATION_TASK_STATUS_MANUAL_CANCELLATION_REQUESTED: MaterializationTaskState
MATERIALIZATION_TASK_STATUS_MANUAL_RETRY: MaterializationTaskState
MATERIALIZATION_TASK_STATUS_PERMANENT_FAILURE: MaterializationTaskState
MATERIALIZATION_TASK_STATUS_RUNNING: MaterializationTaskState
MATERIALIZATION_TASK_STATUS_SUCCESS: MaterializationTaskState
MATERIALIZATION_TASK_STATUS_UNSPECIFIED: MaterializationTaskState

class MaterializationTaskState(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []

class MaterializationTaskAttemptState(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []

class OnlineStoreCopierTaskState(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []
